import 'package:hive/hive.dart';

part 'cart_model.g.dart'; // Generated by Hive

@HiveType(typeId: 0)
class PersistentShoppingCartItem {
  @HiveField(0)
  final String productId;

  @HiveField(1)
  final String productName;

  @HiveField(2)
  final String? productDescription;

  @HiveField(3)
  final String? productThumbnail;

  @HiveField(4)
  final List<String>? productImages;

  @HiveField(5)
  final double unitPrice;

  @HiveField(6)
  int quantity;

  @HiveField(7)
  final Map<String, dynamic>? productDetails;

  @HiveField(8)
  String get key => productId; // Use productId as the key

  PersistentShoppingCartItem({
    required this.productId,
    required this.productName,
    this.productThumbnail,
    this.productDescription,
    this.productImages,
    required this.unitPrice,
    required this.quantity,
    this.productDetails,
  });

  // Add the toJson method to convert PersistentShoppingCartItem to a Map
  Map<String, dynamic> toJson() {
    return {
      'productId': productId,
      'productName': productName,
      'productDescription': productDescription,
      'productThumbnail': productThumbnail,
      'productImages': productImages,
      'unitPrice': unitPrice,
      'quantity': quantity,
      'productDetails': productDetails,
    };
  } // Helper method to calculate the total price for this item

  double get totalPrice => unitPrice * quantity;

  // Helper method to increment the quantity
  void increment() {
    quantity++;
  }

  // Helper method to decrement the quantity
  void decrement() {
    if (quantity > 0) {
      quantity--;
    }
  }
}
